	/* Standard 'C' Includes */
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <dir.h>
#include <dos.h>
#include <string.h>
#include <alloc.h>
#include <ctype.h>
#include <time.h>


	/* User Defined Includes */
#include "genutil.h"
#include "keyasm.h"
#include "keybrd.h"
#include "mouse.h"
#include "buttons.h"
#include "util.h"

	/* Program Specific Includes */
#include "scrlbar.h"
#include "listbox.h"
#include "dragbox.h"



//========== Class DRAG_BUTTON Constructor ==================================
void DRAG_BUTTON::DRAG_BUTTON (const int x1, const int y1, LIST_BOX far *list1, const char *string) {
  x = x1;
  y = y1;
  active = FALSE;
  list = list1;
  title[20] = 0;
  value[0] = 0;
  value[20] = 0;
  number = 0;

  if (string)
    strncpy (title, string, 20);
  else
    title[0] = 0;
 }
//========== End of Class DRAG_BUTTON Constructor ===========================


/*========== Checks Mouse X/Y for Click on a Draglist Button ==============*/
boolean DRAG_BUTTON::check (const int mx, const int my, const int event) {
  if (event != BUTTON_RELEASE) return (FALSE);
  if (mx >= x-5 && mx < x + 3 && my == y) return (TRUE);
  return (FALSE);
 }
/*========== End of Function DRAG_BUTTON::check() =========================*/


/*========== Main Function for 'Running' A Drag-List ======================*/
boolean DRAG_BUTTON::do_draglist (void) {
  boolean redraw = TRUE;	//Need to redraw the drag-box?
  boolean done = FALSE;		//Main loop counter
  boolean cancel = FALSE;	//Were we succesful?
  int orig_textcolor, orig_backcolor;	//For saving screen colors
  int i, event;
  CAPTURE_TEXT cap_text;
  clock_t double_click_time = 0;
  int double_click_item = -1;

  orig_textcolor = get_text_color();	//Save screen colors
  orig_backcolor = get_back_color();

  textcolor (DRAGBOX_BORD_COLOR);	//Save current screen colors
  textbackground (DRAGBOX_BACK_COLOR);
  list->active = TRUE;

	/* Save the background */
  get_screen (list->x - 1, list->y,
	      list->x + list->width + 2, list->y + list->height + 1,
	      &cap_text);

	/* Draw the drag-box */
  draw_box ();

	/* Redraw proper arrow */
  active = TRUE;
  draw_arrow ();

  do {
	/* Redraw the list if we need to */
    if (redraw) {
      redraw = FALSE;
      list->draw ();
      _setcursortype (_NOCURSOR); /* Turn the cursor off */
     }

    event = get_mouse_event (LEFT_BUTTON) | get_mouse_event (RIGHT_BUTTON);

    if (event == BUTTON_RELEASE) {
		/* If a click outside box...exit it */

      if ((i = list->check (MOUSEX, MOUSEY, event)) == NO_ITEM) {
	cancel = TRUE;
	done = TRUE;
       }
      else if (i != SCROLL_BAR) {	/* Is the click in the list? - Return selected name */
	if (list->current + i == double_click_item && clock() - double_click_time <= DOUBLE_CLICK_TIME) {
	  list->move (i);
	  if (list->get_name (NULL)) done = TRUE;
	 }
	else {
	  double_click_time = clock();
	  double_click_item = list->current + i;
	 }

	list->move (i);
	redraw = TRUE;
       }
     }

    GetKey();	/* Get Input from Keyboard */

    if (get_esc_state()) { /* ESC was pressed...exit */
      done = TRUE;
      cancel = TRUE;
     }
    else if (get_enter_state()) { /* Enter Was Pressed */
      if (list->get_name (NULL)) done = TRUE;	/* Exit drag-box */
     }
    else if (get_up_state()) { 	/* Up Cursor Keys */
      if (list->move(-1)) redraw = TRUE;
      delay (KEY_DELAY * 3 / 2);
     } //End of if 'up'
    else if (get_down_state()) { /* Down Cursor Keys */
      if (list->move(1)) redraw = TRUE;
      delay (KEY_DELAY * 3 / 2);
     } //End of if 'down'
    else if (get_pgup_state()) { /* PgUp */
      if (list->move(-list->height)) redraw = TRUE;
      delay (KEY_DELAY * 3 / 2);
     } //End of if 'pgup'
    else if (get_pgdn_state()) { /* PgDn */
      if (list->move(list->height)) redraw = TRUE;
      delay (KEY_DELAY* 3 / 2);
      } //End of if 'pgdn'
    else if (get_home_state()) { /* Home */
      list->view = list->start;
      list->current = 0;
      redraw = TRUE;
      delay (KEY_DELAY);
     }
    else if (get_end_state()) { /* End */
      list->view = list->end;
      list->current = 0;
      redraw = TRUE;
      delay (KEY_DELAY);
     }

  } while (!done);

	/* Replace the screen */
  put_screen (&cap_text);

	/* Redraw proper arrow */
  active = FALSE;
  list->active = FALSE;
  draw_arrow ();

	/* Reset screen colors */
  textcolor (orig_textcolor);
  textbackground (orig_backcolor);

  if (!cancel && list->list && list->list->data) {
    _fstrncpy (value, list->list->data, 20);
    number = get_number (list->list);
   }

  return ((boolean) !cancel);
 }
/*========== End of function do_draglist() ================================*/


/*========== Draws a Singular Drag-Type Box on the Screen =================*/
void DRAG_BUTTON::draw_box (void) {
  int count;	/* Loop counter */
  int x1, y1, width, height; 	/* Dimensions of Box */
  char buffer[81] = "01234567890123456789012345678901234567890123456789012345678901234567890123456789";

	/* Set the boxes coordinates */
  x1 = list->x - 1;
  y1 = list->y;
  width = list->width + 4;
  height = list->height + 1;

	/* Define a string for the 'middle' parts...for speed */
  buffer[width] = 0;
  strset (buffer, ' ');
  buffer[0] = SING_VERTICAL;
  buffer[width - 1] = SING_VERTICAL;

  hide_mouse();

  for (count = y1; count < y1 + height - 1; count++) {
    gotoxy (x1, count);
    cprintf ("%s", buffer);
   }

  gotoxy (x1, y1 + height - 1);
  cprintf ("%c", BOTLEFT_SING_CORN);

  for (count = x1 + 1; count < x1 + width - 1; count++)
    cprintf ("%c", SING_HORIZONTAL);

  cprintf ("%c", BOTRIGHT_SING_CORN);

  show_mouse();
 }
/*========== End of procedure draw_dragbox() ==============================*/


/*========== Draws all the little nice arrows for all the Dragboxes =======*/
void DRAG_BUTTON::draw_arrow (void) {
  int orig_textcolor, orig_backcolor;	/* For saving screen colors */

  orig_textcolor = get_text_color();	/* Save screen colors */
  orig_backcolor = get_back_color();

  hide_mouse ();

	/* Set proper colors for drawing arrows */
  textcolor (DRAGBOX_ARROW_TEXT_COLOR);
  textbackground (DRAGBOX_ARROW_BACK_COLOR);

	/* Draw the arrow */
  gotoxy (x, y);

  if (active)
    cprintf ("%c", 24);
  else
    cprintf ("%c", 25);

	/* Reset screen colors */
  textcolor (orig_textcolor);
  textbackground (orig_backcolor);

  show_mouse ();
 }
//========== End of Procedure draw_draglist_arrows() ========================


/*========== Draws the Selected Value...If any ============================*/
void DRAG_BUTTON::draw_text (void) {
  int orig_textattr = get_textattr();

  hide_mouse();
  textcolor (LIGHTGRAY);
  textbackground (BLACK);
  gotoxy (x - list->width - 3, y);
  cprintf (" %-*s ", list->width + 1, value);
  textattr (orig_textattr);
  show_mouse();
 }
/*========== End of procedure DRAG_BUTTON::draw_text() ====================*/


/*========== Draws the Selected Value Title ===============================*/
void DRAG_BUTTON::draw_title (void) {
  hide_mouse();
  gotoxy (x - list->width - 4 - strlen(title), y);
  cprintf ("%s", title);
  show_mouse();
 }
/*========== End of procedure DRAG_BUTTON::draw_title() ===================*/


/*========== Finds a Associated Extended Value ============================*/
void DRAG_BUTTON::find_ext_char (const unsigned char ch) {
  DLINKED_LIST far *temp_list;

  if (!list || !list->start) return;
  temp_list = list->start;

  while (temp_list) {
    if (ch == get_ext_char(temp_list)) {
      _fstrncpy (value, temp_list->data, 20);
      number = get_number (temp_list);
      return;
     }

    temp_list = temp_list->next;
   }

  number = get_number (list->end);
  _fstrncpy (value, list->end->data, 20);
 }
/*========== End of procedure DRAG_BUTTON::find_ext_char() ================*/


/*========== Finds a Associated Extended Value ============================*/
void DRAG_BUTTON::find_ext_int (const unsigned int i) {
  DLINKED_LIST far *temp_list;

  if (!list || !list->start) return;
  temp_list = list->start;

  while (temp_list) {
    if (i == get_ext_int(temp_list)) {
      _fstrncpy (value, temp_list->data, 20);
      number = get_number (temp_list);
      return;
     }

    temp_list = temp_list->next;
   }

  number = get_number (list->end);
  _fstrncpy (value, list->end->data, 20);
 }
/*========== End of procedure DRAG_BUTTON::find_ext_int() =================*/


/*========== Finds a Associated Extended Value ============================*/
void DRAG_BUTTON::find_ext_long (const unsigned long l) {
  DLINKED_LIST far *temp_list;

  if (!list || !list->start) return;
  temp_list = list->start;

  while (temp_list) {
    if (l == get_ext_long(temp_list)) {
      _fstrncpy (value, temp_list->data, 20);
      number = get_number (temp_list);
      return;
     }

    temp_list = temp_list->next;
   }

  number = get_number (list->end);
  _fstrncpy (value, list->end->data, 20);
 }
/*========== End of procedure DRAG_BUTTON::find_ext_long() ================*/



/*========== Adds Button To End of Array ==================================*/
int DRAG_ARRAY::add_button (const int x1, const int y1, LIST_BOX far *list1, const char *string) {
	/* Returns index if button added successfully, or -1 if array is
	 * full. End of array is defined as 1st entry with x=-1 or last entry */

  int i;	/* Loop counter */

  for (i = 0; i < MAX_DRAG_ARRAY && buttons[i].x != -1; i++) {
   }

  if (i == MAX_DRAG_ARRAY) return (-1);
  if (i != MAX_DRAG_ARRAY - 1) buttons[i + 1].x = -1;
  buttons[i].x = x1;
  buttons[i].y = y1;
  buttons[i].list = list1;

  if (string)
    strncpy (buttons[i].title, string, 20);
  else
    buttons[i].title[0] = 0;

  return (i);
 }
//========== End of function add_button() ===================================


//========== Adds Button To End of Array ====================================
int DRAG_ARRAY::add_button (LIST_BOX far *list1, const char *string) {
	/* Returns index number of if button added successfully, or -1 if array is
	 * full. End of array is defined as 1st entry with x=-1 or last entry */

  return (add_button (list1->x + list1->width + 2, list1->y - 1, list1, string));
 }
//========== End of function add_button() ===================================


//========== Checks all Drag Buttons in Array and Calls Appropiate Routine ==
boolean DRAG_ARRAY::check (const int mx, const int my, const int event) {
  int i;	/* Loop counter */

  if (event != BUTTON_RELEASE) return (FALSE);

  for (i = 0; i < MAX_DRAG_ARRAY && buttons[i].x != -1; i++) {
    if (buttons[i].check (mx, my, event)) return (buttons[i].do_draglist());
   }

  return (FALSE);
 }
//========== End of Function DRAG_ARRAY::check() ============================


/*========== Draws everything =============================================*/
void DRAG_ARRAY::draw_all (void) {
  draw_titles();
  draw_texts();
  draw_arrows();
 }
/*========== End of procedure DRAG_ARRAY::draw_all() ======================*/


//========== Draws all Arrow Buttons in Array ===============================
void DRAG_ARRAY::draw_arrows (void) {
  int i;	/* Loop counter */

  for (i = 0; i < MAX_DRAG_ARRAY && buttons[i].x != -1; i++) {
    buttons[i].draw_arrow ();
   }
 }
//========== End of Procedure draw_arrows() =================================


/*========== Draws all the Selected Value...If any ========================*/
void DRAG_ARRAY::draw_texts (void) {
  int i;	/* Loop counter */

  for (i = 0; i < MAX_DRAG_ARRAY && buttons[i].x != -1; i++) {
    buttons[i].draw_text();
   }
 }
/*========== End of procedure draw_texts() ================================*/


/*========== Draws the Selected Value Title ===============================*/
void DRAG_ARRAY::draw_titles (void) {
  int i;	/* Loop counter */

  for (i = 0; i < MAX_DRAG_ARRAY && buttons[i].x != -1; i++) {
    buttons[i].draw_title();
   }
 }
/*========== End of procedure DRAG_BUTTON::draw_title() ===================*/


/*========== Initializes the Values =======================================*/
void DRAG_ARRAY::init_values (void) {
  int i;	/* Loop counter */

  for (i = 0; i < MAX_DRAG_ARRAY && buttons[i].x != -1; i++) {
    if (buttons[i].list && buttons[i].list->start && buttons[i].list->start->data) {
      _fstrncpy (buttons[i].value, buttons[i].list->start->data, 20);
      buttons[i].number = get_number (buttons[i].list->start);
     }
    else {
      _fstrcpy (buttons[i].value, "<none>");
      buttons[i].number = 0;
     }
   }
 }
/*========== End of procedure DRAG_ARRAY::init_values() ===================*/


/*========== Gets a Number from a Dual-Linked List ========================*/
unsigned long get_number (DLINKED_LIST far *dllist) {
  char far *temp_ptr;

  if (!dllist || !dllist->data)
    return(0);

  temp_ptr = dllist->data + _fstrlen(dllist->data) + 1;

  if (*temp_ptr) {
    if (_fstrlen(temp_ptr) == 1)
      return((long) *temp_ptr);
    else
      return (strtoul (temp_ptr, NULL, 0) );
   }
  else
    return (0);

 }
/*========== End of procedure get_number() ================================*/