/*===========================================================================
 *				BEGIN 'DEFINES'
 *=========================================================================*/

/* Some modified conversion routines I originally found from some BMP conversion
 * code. These convert a character string (such as read from a file) and converts
 * it to a more familiar base 10 integer (either long or short). This is useful
 * since the bytes for integers in a file are reversed and I don't trust scanf()
 * to do what I want. I'd rather read 2/4 characters and use these routines */
#define HexToLong(Start) (unsigned long)(*(Start) & 0xff) + \
			 (unsigned long)((*(Start + 1) << 8) & 0xffff) + \
			 (unsigned long)(((unsigned long)*(Start+3) << 24) & 0xffffffffl) + \
			 (unsigned long)(((unsigned long)*(Start+2) << 16) & 0xffffffl)

#define HexToInt(Start) (unsigned int)(*(Start) & 0xff) + \
			(unsigned int)((*(Start + 1) << 8) & 0xffff)


/* A little function to compare the equality of two strings. Returns
 * TRUE if equal, FALSE otherwise */
#define ISEQ(string, target) (!_fstrnicmp(string, target, _fstrlen(target)) )


/* Define the argument for interrupt type prototypes. These arguments are
 * different in 'C' than in 'C++'. I use C++ all the time now, so I don't
 * really need this */
#ifdef __cplusplus
    #define __CPPARGS ...
#else
    #define __CPPARGS
#endif


/* If you're really used to BASIC, pascal etc..., use these */
#define AND &&
#define and &&
#define OR ||
#define or ||


	/* Return values for the get_yesno(), get_yesnoall() type function */
#define CH_YES 0
#define CH_NO  1
#define CH_ESC 2
#define CH_ALL 3


/* Define some common error messages to save space...I hope. I suppose this
 * only saves space if your compiler has a 'merge duplicate strings' option
 * as does Turbo C/C++ v3.0. It certainly makes for less typing and is much
 * easier too. Another alternative would be to have these as const character
 * arrays...shrug  */
#define FILE_ERR_MSG "Could not open file!"
#define MEM_ERR_MSG "Could not allocate memory for pointer!"

	/* For new DOS error handler */
#define DOS_ERR_IGNORE  -1
#define DOS_ERR_RETRY   -2
#define DOS_ERR_ABORT   -3

	/* Some types derived from ASM */
#define WORD int
#define BYTE unsigned char

	/* Defines for replacing the timer interrupt. Taken from the book
	 * 'Teach yourself Game Programming in 21 days' */
#define TIME_KEEPER_INT     0x1C    /* the time keeper interrupt */
#define CONTROL_8253        0x43    /* the 8253 control register */
#define CONTROL_WORD        0x3C    /* the control word to set mode 2, binary least/most */
#define COUNTER_0           0x40    /* 'Codes' for the various timer interrupts */
#define COUNTER_1           0x41
#define COUNTER_2           0x42


	/* Some commone Error Codes I use for my error/bug handling routines
	 * The numbers coorespond to the index in the char *err_desc[] string
	 * array */
#define ERR_OVERREAD 0		/* Read past end of file */
#define  ERR_INVALID 1          /* Invalid file type */
#define ERR_NOTENOUGH 2		/* Didn't read/get enough of something */
#define ERR_ARRAY_OVERFLOW 3 	/* Array Bounds overflow error */


	/* WORD conversion routines. I forget where I got these, but they
	 * simply get the high and low bytes out of an integer (kinda like
	 * using al, ah to access the high and low bytes of the ax register
	 * in assembly...which this isn't) */
#define LOW_BYTE(n)  (n & 0x00ff)       /* Extracts the low-byte */
#define HIGH_BYTE(n) ((n>>8) & 0x00ff)  /* Extracts the high-byte */


	/* Maximum number of tasks allowable. This is used as the size for
	 * the task_list[] array. Having 20 tasks being executed in a semi-
	 * multitasking environment should be enough to fry most anyones thinker,
	 * including most processors. If you have a lot of small, fast, routines
	 * which you need to be excuted by the timer interrupt, increase this */
#define MAX_TASKS 20

	/* The maximum allowed string length used in GENUTIL.CPP */
#define GENUTIL_STRING_LENGTH 101

	/* Characters to draw line for boxes and buttons etc... */
#define        HORIZONTAL 205
#define          VERTICAL 186
#define   TOPRIGHT_CORNER 187
#define   BOTRIGHT_CORNER 188
#define    TOPLEFT_CORNER 201
#define    BOTLEFT_CORNER 200
#define           T_RIGHT 199
#define            T_LEFT 182
#define  TOPLEFT_SING_CORN 218
#define  BOTLEFT_SING_CORN 192
#define TOPRIGHT_SING_CORN 191
#define BOTRIGHT_SING_CORN 217
#define    SING_HORIZONTAL 196
#define      SING_VERTICAL 179


	/* Define some things needed for the menu command structure */
#if defined(TRADITIONAL)
  #define const
  #define args( list )			( )
  #define DECLARE_DO_COMMAND( command )	void command( )
#else
  #define args( list )			list
  #define DECLARE_DO_COMMAND( command )	DO_COMMAND    command
#endif



/*===========================================================================
 * 			END of 'DEFINES'
 *=========================================================================*/


	/* Define the standard boolean type...very useful for loops etc...
	 * (I used to use PASCAL a lot and I missed the boolean type) To
	 * avoid those pesky errors relating to converting integers to
	 * enum's and such, simply use typecasts */
typedef enum {FALSE=0, TRUE=1} boolean;


	/* Another thing needed for the command structure to work */
typedef	boolean DO_COMMAND args( (char *argument ) );


/*===========================================================================
 * 			BEGIN Class/Structure Definitions
 *=========================================================================*/

#if __cplusplus
/*========== Multi-Tasking Type by Reprogramming Internal Timer ===========
 * Used for calling tasks from the new timer interrupt. Use the add_task()
 * and delete_task() to add/delete tasks */
class TASK_TYPE {

public:
	/* Are we processing this state currently? Allows use to 'pause' some
	 * tasks while still performing others. Thus, you can pause the main
	 * game tasks (such as animation control) while still using that useful
	 * blinking cursor task */
  boolean active;

	/* Counter value which triggers task. Often I don't always want to
	 * execute a task each time the timer interrupt is called (by default
	 * this is 18.2 times each second). Slower routines which don't need
	 * that frequency of being called can be called every so often. Thats
	 * what the max_count and count variables do. If count >= max_count the
	 * event is executed and count is reset to 0. If count < max_count, count
	 * is simply incremented and the task is NOT executed. Thus, a max_count of
	 * '0' means the task is performed on each call of the timer interrupt,
	 * while a value of '1' means every other call...etc */
  unsigned max_count;
  unsigned count;

	/* The pointer to the task procedure. The procedure must be defined
	 * with no arguments and returns nothing (ie: void func (void) {}). If
	 * there is away around this...please tell me. I'd like to be able
	 * to pass arguments...then I could create a general color_rotation
	 * procedure which accepts a pointer to a color varaible...instead of
	 * different color rotation procedures for each different application */
  void far (*task)(__CPPARGS);

	/* Class Constructor...sets values to some initial value */
  TASK_TYPE (void);
 };
/*========== End of structure TASK_TYPE definition ========================*/
#endif


/*========== The command type structure, useful for creating a command line
 *========== parameter shell etc... =======================================*/
typedef struct {
  char *name;
  DO_COMMAND *do_command;
 } COMMAND_TYPE;
/*========== End of Class COMMAND_TYPE Definition =========================*/


/*========== Class CAPTURE_TEXT Definition ================================*/
	/* Makes the capturing of a portion of the text screen easier */
typedef struct {
  char *save_screen;	/* The pointer for saving the screen to */
  int x, y, x1, y1;	/* Coordinates of the save screen */
 } CAPTURE_TEXT;

/*========== End of Class CAPTURE_TEXT Definition =========================*/




/*===========================================================================
 * 			END Class/Structure Definitions
 *=========================================================================*/


/*===========================================================================
 * 		      BEGIN External Variable Definitions
 *
 * See the file GENUTIL.CPP for a description of each variable...I hope. If
 * it's not there, you're stuck...;)
 *=========================================================================*/
extern char far *err_header_string;
extern char far *err_header_version;
extern unsigned err_code, ext_err_code;
extern char far *err_desc[];
extern unsigned max_string_length;
extern boolean do_tasks;

#if __cplusplus
extern TASK_TYPE task_list[];
#endif

/*===========================================================================
 * 		      END External Variable Definitions
 *=========================================================================*/




/*===========================================================================
 * 		      Function and Procedure Prototypes
 *
 * For more information on these routines, see the file GENUTIL.CPP, as well
 * as possibly any struct/class used
 *=========================================================================*/

	/* The original Timer Interrupt saved by set_timer_int and reset
	 * by remove_timer_int */
extern void interrupt (*Old_Time_Isr)(__CPPARGS);

	/* Adds a task to the end of task list, returning index number on
	 * success, or -1 or failure */
int add_task (void (far *function)(__CPPARGS), const unsigned max_counter);

	/* General Error handling routine */
void bug(char *msg, char *string, ...);

	/* Checks to see if there is any disk in drive */
boolean check_disk (const int drive);

	/* Attempts to Create a char pointer. Returns pointer on success
	 * and exits with an error on failure. ptr is optional parameter
	 * and should be the pointer being allocated.  This is used to
	 * check if the pointer is already allocated or not (non-NULL
	 * pointers are assumed to be allocated).  If it is non-NULL
	 * it deletes the ptr. */
char *create_ptr (const char *string, char *ptr = NULL);
char *create_ptr (const unsigned l, char *ptr = NULL);

	/* Attempts to Create a far pointer */
char far *create_farptr (const char far *string, char far *ptr = NULL);
char far *create_farptr (const unsigned l, char *ptr = NULL);


	/* Prints Text Center on the Screen...by some y-offset */
int cprintf_cent (const int y_offset, const char far *format, ...);

	/* Adds one character to end of string...doesn't check for length
	 * of string to be < MAX_STRING_LENGTH */
void chrcat(char far *buf, const char ch);

	/* Deletes a task from the task_list array, index i (ie task_list[i]) */
void delete_task (unsigned i);

	/* Returns Current Background Color in Text Mode */
int get_back_color(void);

	/* Returns the background color at a screen location */
int get_char_backcolor(const int x, const int y);

	/* Returns the text color at a screen location */
int get_char_textcolor(const int x, const int y);

	/* Returns TRUE if user presses ENTER. FALSE if input exceeds msl characters
	 * or if ESC was pressed. msl = max_string_length if not specified */
boolean get_line (char far *buffer, const int msl = max_string_length);

	/* Returns the Height in Characters of current text screen */
int get_screenheight(void);

	/* Returns the Width in Characters of current text screen */
int get_screenwidth(void);

	/* Returns Current Text Color in Text Mode */
int get_text_color(void);

	/* Returns Current Text Attribute in Text Mode */
int get_textattr(void);

	/* Gets a Yes/No type answer from the keyboard */
int get_yesno (void);

	/* Gets a Yes/No/All type answer from the keyboard */
int get_yesnoall (void);

	/* Converts an integer to a Hexadecimal string */
void int_to_hex (const unsigned int i, char *buf);

	/* Is a Point (x,y) in a particular Rectangle? */
boolean iscontained(int x, int y, int x1, int y1, int width, int height);

	/* Returns TRUE if the character is a printable one. Printable is
	 * defined as between the ' ' (space) and the '~' (tilde) characters */
boolean isprintable(const char ch);

	/* Returns TRUE if the character is a punctuation type character.
	 * ',.?"[];:' etc... */
boolean ispunctuation(const unsigned char ch);

	/* Converts long integer  to Hexdecimal string */
void long_to_hex (const unsigned long i, char *buf);

	/* Strips Space and TAB characters from Left of String Returns
	 * Pointer to 1st non-space/tab character in string */
char *lstrip (char *string);

	/* Returns maximum of the two values */
unsigned maximum (unsigned value1, unsigned value2);

	/* Parses a Command Line Command */
boolean parse_command (char *string, COMMAND_TYPE *cmd_table);

	/* Prints a comma delimited number */
void print_number (const unsigned long num);

	/* Returns TRUE if y-range <= x <= y + range */
boolean range (const int y, const int range, const int x);

	/* Replaces the user define timer interrupt with the original
	 * timer interrupt saved in Old_Time_Isr() */
void remove_timer_int(void);

	/* Strips Space and TAB characters from Right of String Returns
	 * Pointer to string */
char *rstrip (char *string);

	/* Sets the new timer interrupt function to be New_Time_Isr.  Used
	 * for animations mostly */
void set_timer_int(void interrupt (*New_Time_Isr)(__CPPARGS));

	/* Returns the sign of the Expression (-1/0/1) */
int sign (const int i);

	/* Define our procedure which will replace the default
	 * timer interrupt function.  This will control animation
	 * counters and possibly other things */
void interrupt far Timer(__CPPARGS);
/*===========================================================================
 * 		      END Function and Procedure Prototypes
 *=========================================================================*/

