
	/* Standard 'C' Includes */
#include <stdlib.h>
#include <conio.h>
#include <stdio.h>
#include <string.h>
#include <alloc.h>

	/* User Defined Includes */
#include "genutil.h"
#include "fileutil.h"
#include "mouse.h"
#include "util.h"
#include "scrlbar.h"

	/* The equation for computing the y-coordinate of the value button */
#define FIND_Y(my, val) (my + (length - 3) * (val - min) / (max - min) + 1 )

	/* The inverse equation for above...computes value from a Y-coordinate */
#define FIND_VALUE(my, ry) (my - 8*(ry - 1)) * (max - min) / (8*(length - 3)) + min


/*========== Class SCROLL_TYPE Constructor ================================*/
void SCROLL_TYPE::SCROLL_TYPE(void) {
  x = y = y50 = 1;
  length = 3;
  min = 0;
  max = 0;
  value = 0;
  pressed = FALSE;
  text_color = SCROLL_TEXT_COLOR;
  back_color = SCROLL_BACK_COLOR;
  visible = FALSE;
 }
/*========== End of Function void SCROLL_TYPE::SCROLL_TYPE(void) ==========*/


/*========== Class SCROLL_TYPE Constructor ================================*/
void SCROLL_TYPE::SCROLL_TYPE(const char x1, const char y1, const char y501, const char l,
			      const int minimum, const int maximum,
			      const char bcolor, const char tcolor) {
  x = x1;
  y = y1;
  y50 = y501;
  length = l;
  min = minimum;
  max = maximum;
  value = min;
  text_color = tcolor;
  back_color = bcolor;
  visible = TRUE;
 }
/*========== End of Function void SCROLL_TYPE::SCROLL_TYPE() ==============*/


/*========== Changes the Maximum Value ====================================*/
void SCROLL_TYPE::change_max (const int how_much) {
  if (max + how_much < min) {
    max = min;
    value = min;
   }
  else
    max += how_much;

  if (value > max) value = max;
  update_button();
 }
/*========== End of procedure SCROLL_TYPE::change_max() ===================*/


/*========== Changes the Minimum Value ====================================*/
void SCROLL_TYPE::change_min (const int how_much) {
  if (min + how_much > max) {
    min = max;
    value =  min;
   }
  else
    min += how_much;

  if (value < max) value = min;
  update_button();
 }
/*========== End of procedure SCROLL_TYPE::change_max() ===================*/


/*========== Changes the Value ============================================*/
void SCROLL_TYPE::change_value (int new_value) {
  int old_y, new_y, y1;

	/* Is the new value valid? */
  if (min == max)
    return;
  else if (new_value < min)
    new_value = min;
  else if (new_value > max)
    new_value = max;

  if (value == new_value) return;

	/* Use correct screen y-coordinate */
  if (get_screenheight() > 25)
    y1 = y50;
  else
    y1 = y;

	/* Compute position of button */
  old_y = FIND_Y(y1, value);
  new_y = FIND_Y(y1, new_value);
  value = new_value;

	/* Draw the button */
  if (old_y != new_y) {
    if (new_y == y1 + 1)
      value = min;
    else if (new_y == y1 + length)
      value = max;

    update_button();
   }
 }
/*========== End of procedure SCROLL_TYPE::change_value() =================*/


/*========== Checks to see if a Mouse was clicked on it ===================*/
int SCROLL_TYPE::check (const int mx, const int my, const int event) {
  int y1, i, pre_value = value;

  if (event == NO_EVENT && !pressed) return (NO_SCROLL);

  if (get_screenheight() > 25)
    y1 = y50;
  else
    y1 = y;

  if (pressed || (mx == x && my >= y1 && my <= y1 + length - 1)) {
    if (max != min)	/* Current location of the button */
      i = FIND_Y(y1, value);
    else
      i = y1 + 1;

    if (my == i && event == BUTTON_PRESS)
      pressed = TRUE;
    else if (pressed && event == BUTTON_RELEASE) {
      pressed = FALSE;
      return (NO_SCROLL);
     }

    if (pressed) { 	/* Value button */
      get_mouse_pos();

      if (length > 3)
	value = FIND_VALUE(MouseY, y1);
      else
	return (NO_SCROLL);

      if (value > max)
	value = max;
      else if (value < min)
	value = min;

      return (value - pre_value);
     }

    if (event != BUTTON_RELEASE) return (NO_SCROLL);

    if (my == y1)	/* Up button */
      change_value (value - 1);
    else if (my == y1 + length - 1) 	/* Down button */
      change_value (value + 1);
    else if (my < i) 	/* On the scroll bar...pageup */
      change_value (value - length + 1);
    else if (my > i) 	/* On the scroll bar...pagedown */
      change_value (value + length - 1);

    return (value - pre_value);
   }

  return (NO_SCROLL);
 }
/*========== End of Function SCROLL_TYPE::check() =========================*/


/*========== Draws a Scroll bar on the screen =============================*/
void SCROLL_TYPE::draw() {
  int orig_textattr = get_textattr();	/* Save the current screen colors */
  int y1, i;

  if (!visible) return;

	/* Set the screen y-coordinate of bar based on screen mode */
  if (get_screenheight() > 25)
    y1 = y50;
  else
    y1 = y;

  hide_mouse();
  textcolor (text_color);
  textbackground (back_color);

	/* Print the up/down arrows */
  gotoxy (x, y1);
  cprintf ("%c", SCROLL_UP_ARROW);
  gotoxy (x, y1 + length - 1);
  cprintf ("%c", SCROLL_DOWN_ARROW);

	/* Print the spaces...if any */
  if (length > 3) {

    for (i = y1 + 1; i < y1 + length - 1; i++) {
      gotoxy (x, i);
      cprintf ("%c", SCROLL_SPACE);
     }
   }

	/* Draw the button */
  update_button();

	/* Restore screen colors */
  textattr(orig_textattr);
  show_mouse();
 }
/*========== End of Function void SCROLL_TYPE::draw() =====================*/


/*========== Resets the Scroll Bar to Contain Nothing =====================*/
void SCROLL_TYPE::reset (void) {
  min = max = value = 0;
  draw();
  visible = FALSE;
 }
/*========== End of procedure SCROLL_TYPE::reset() ========================*/


/*========== Updates the button position ==================================*/
void SCROLL_TYPE::update_button (void) {
  int orig_textattr = get_textattr();	/* Save the current screen colors */
  int y1;

  if (!visible) return;

	/* Set the screen y-coordinate of bar based on screen mode */
  if (get_screenheight() > 25)
    y1 = y50;
  else
    y1 = y;

  hide_mouse();
  textcolor (text_color);
  textbackground (back_color);

	/* Print the button...if any */
  if (length > 2) {
    if (value == min)
      gotoxy (x, y1 + 1);
    else if (min != max)
      gotoxy (x, FIND_Y(y1, value));
    else
      gotoxy (x, y1 + 1 );

    cprintf ("%c", SCROLL_BUTTON);
   }

	/* Restore screen colors */
  textattr(orig_textattr);
  show_mouse();
 }
/*========== End of Function void SCROLL_TYPE::update_button ==============*/


