	/* Standard Includes */
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <conio.h>
#include <string.h>
#include <ctype.h>
#include <sys\stat.h>
#include <dos.h>
#include <dir.h>
#include <direct.h>
#include <fcntl.h>

	/* User Defined Includes */
#include "genutil.h"
#include "keybrd.h"
#include "keyasm.h"
#include "mouse.h"
#include "buttons.h"
#include "util.h"


	/* Define buttons for yes/no dialogue box */
BUTTON_ARRAY yes_no_buttons;

	/* Define buttons for retry/abort/ignore DOS error box */
BUTTON_ARRAY dos_err_buttons;

	/* Define single button for error dialogue box */
BUTTON_TYPE ok_button;

	/* Define the error messages for trapping disk problems */
char dos_err_msg[16][20] = {
     "write protect",
     "unknown unit",
     "drive not ready",
     "unknown command",
     "data error (CRC)",
     "bad request",
     "seek error",
     "unknown media type",
     "sector not found",
     "printer out of paper",
     "write fault",
     "read fault",
     "general failure",
     "reserved",
     "reserved",
     "invalid disk change"
 };



/*========== Replaces the default DOS ctrl-brk handler ====================*/
int ctrl_brk_handler (void) {
  if (get_yn_choice(30, 10, "CTRL-BRK Pressed", "Do you Wish to Terminate the Program"))
    return (1);	//Don't abort

  return (0);	//Abort Program
 }
/*========== End of function crtl_brk_handler() ===========================*/


/*========== Destorys the Default Button Arrays ===========================*/
void destroy_buttons (void) {
  yes_no_buttons.destroy();
  dos_err_buttons.destroy();;
  ok_button.destroy();;
 }
/*========== End of procedure destroy_buttons() ===========================*/


/*========== Error Handler for Trapping DOS errors (like no disk in A) ====*/
int dos_error_win (const char *msg) {
  int orig_text_color, orig_back_color;	/* Save current color settings */
  int return_value = DOS_ERR_ABORT;	/* What we want to return */
  CAPTURE_TEXT cap_text;   	/* For saving screen background */
  boolean done = FALSE;		/* Main loop counter */

	/* Save current screen colors */
  orig_text_color = get_text_color();
  orig_back_color = get_back_color();

	/* Save background */
  get_screen (strlen(msg) + 13, 11, &cap_text);

	/* Set colors for error box */
  textcolor (ERROR_BORD_COLOR);
  textbackground (ERROR_BACK_COLOR);
  draw_textbox_center (strlen(msg) + 13, 11, "DOS Error", ERROR_TEXT_COLOR);

  textcolor (ERROR_TEXT_COLOR);
  hide_mouse();
  cprintf_cent (-3, "%s", msg);
  cprintf_cent (-1, "Retry/Abort/Ignore [r/a/i]?");
  show_mouse();

  dos_err_buttons.draw ();

	/* Get/Parse Input Loop */
  do {
    switch (dos_err_buttons.check (MOUSEX, MOUSEY, get_mouse_event (LEFT_BUTTON)) ) {
	case 0: done = TRUE;	/* Retry button pressed */
		return_value = DOS_ERR_RETRY;
		break;
	case 1: done = TRUE;	/* Abort button pressed */
		return_value = DOS_ERR_ABORT;
		break;
	case 2: done = TRUE;	/* Ignore button pressed */
		return_value = DOS_ERR_IGNORE;
		break;
     } /* End of switch which button pressed */

    GetKey();	/* Get keyboard input */
    keylast = toupper (keylast);

    if (keylast == 'R') {
      done = TRUE;
      return_value = DOS_ERR_RETRY;
     }
    else if (keylast == 'A') {
      done = TRUE;
      return_value = DOS_ERR_ABORT;
     }
    else if (keylast == 'I') {
      done = TRUE;
      return_value = DOS_ERR_IGNORE;
     }
  } while (!done);

	/* Reset The Original Screen colors */
  textcolor (orig_text_color);
  textbackground (orig_back_color);

	/* Reset the screen background */
  put_screen (&cap_text);

  return (return_value);
 }
/*========== End of function dos_error_win() ==============================*/


/*========== Our handler which Replaces the DOS error handler =============*/
int dos_err_handler (int errval, int ax, int bp, int si) {
  char msg[MAX_STRING_LENGTH];	/* Temp string buffer */
  unsigned di;
  int drive;
  int errorno;

  di = _DI;

	/* If this is not a disk error then it was another device having
	 * trouble */
  if (ax < 0) {
    dos_error_win ("Device error?");	/* Report the error */
    hardretn (DOS_ERR_ABORT);	        /* and return to the program directly requesting abort */
   }

	 /* Otherwise it was a disk error */
  drive = ax & 0x00FF;
  errorno = di & 0x00FF;

	/* Report which error it was */
  sprintf(&msg[0], "Error: %s on drive %c",
	  dos_err_msg[errorno], 'A' + drive);

	/* Return to the program via dos interrupt 0x23 with abort, retry,
	 * or ignore as input by the user. */
  hardretn (dos_error_win(msg));
  return (DOS_ERR_ABORT);
 }
/*========== End of function dos_err_handler() ============================*/


/*========== Draws a textbox on the screen ================================*/
void draw_textbox (const int x, const int y, const int x1, const int y1, const char *string, const int text_color) {
  int count;
  char buffer[81] = "01234567890123456789012345678901234567890123456789012345678901234567890123456789";

  buffer[x1 - x + 1] = '\0';     /* Define a string for the 'middle' parts */
  strset (buffer, ' ');
  buffer[0] = VERTICAL;
  buffer[x1 - x] = VERTICAL;

  hide_mouse();

  gotoxy (x, y);
  cprintf ("%c", TOPLEFT_CORNER);

  for (count = x + 1; count < x1; count++)
    cprintf ("%c", HORIZONTAL);

  cprintf ("%c", TOPRIGHT_CORNER);

  for (count = y + 1; count < y1; count++) {
    gotoxy (x, count);
    cprintf ("%s", buffer);
   }

  gotoxy (x, y1);
  cprintf ("%c", BOTLEFT_CORNER);

  for (count = x + 1; count < x1; count++)
    cprintf ("%c", HORIZONTAL);

  cprintf ("%c", BOTRIGHT_CORNER);

  if (string) { /* Display Text Header in Center of Top Line */
    cprintf_cent (y - (get_screenheight() + 1)/2, "%c %*s %c", 181, strlen(string), " ", 198);
    textcolor (text_color);
    cprintf_cent (y - (get_screenheight() + 1)/2, "%s", string);
   }

  show_mouse();
 }
/*========== End of procedure draw_textbox() ==============================*/


/*========== Draws a textbox on the screen with single border =============*/
void draw_sing_textbox (const int x, const int y, const int x1, const int y1, const char *string, const int text_color) {
  int count;
  char buffer[81] = "01234567890123456789012345678901234567890123456789012345678901234567890123456789";

  buffer[x1 - x + 1] = '\0';     //Define a string for the 'middle' parts
  strset (buffer, ' ');
  buffer[0] = SING_VERTICAL;
  buffer[x1 - x] = SING_VERTICAL;

  hide_mouse();

  gotoxy (x, y);
  cprintf ("%c", TOPLEFT_SING_CORN);

  for (count = x + 1; count < x1; count++)
    cprintf ("%c", SING_HORIZONTAL);

  cprintf ("%c", TOPRIGHT_SING_CORN);

  for (count = y + 1; count < y1; count++) {
    gotoxy (x, count);
    cprintf ("%s", buffer);
   }

  gotoxy (x, y1);
  cprintf ("%c", BOTLEFT_SING_CORN);

  for (count = x + 1; count < x1; count++)
    cprintf ("%c", SING_HORIZONTAL);

  cprintf ("%c", BOTRIGHT_SING_CORN);

	/* Display Text Header in Center of Top Line */
  if (string) {
    count = (x1 + x - strlen(string) - 4)/2;
    if (count < 2) count = 2;
    gotoxy(count, y);
    cprintf("%c %*s %c", 181, strlen(string), " ", 198);
    textcolor (text_color);
    gotoxy(count + 2, y);
    cprintf("%s", string);
   }

  show_mouse();
 }
/*========== End of procedure draw_sing_textbox() =========================*/


/*========== Draws a textbox Center in the screen =========================*/
void draw_textbox_center (const int width, const int height,
			  const char *string, const int text_color) {
  struct text_info ti;

  gettextinfo(&ti);
  draw_textbox ((ti.screenwidth-width)/2 + 1, (ti.screenheight-height)/2 + 1,
		(ti.screenwidth+width)/2, (ti.screenheight+height)/2,
		 string, text_color);
 }
/*========== End of procedure draw_textbox_center() =======================*/


/*========== Displays an Error Type box ===================================*/
void error_box (const char *title, const char *msg, ...) {
  int orig_text_color, orig_back_color;	/* Save current color settings */
  CAPTURE_TEXT cap_text;	/* For saving screen background */
  char *buffer;		/* Temp buffer for holding printed string */
  va_list arg_list;	/* For displaying the variable argument list */

	/* Allocated temp buffer and print msg string to it */
  if (!(buffer = new char [MAX_STRING_LENGTH + 1]))
    bug (MEM_ERR_MSG, "error_box() - *buffer (%d)", MAX_STRING_LENGTH);

  va_start (arg_list, msg);
  vsprintf (buffer, msg, arg_list);
  va_end (arg_list);

	/* Save background */
  get_screen (strlen(buffer) + 6, 10, &cap_text);

  orig_text_color = get_text_color();
  orig_back_color = get_back_color();

  textcolor (ERROR_BORD_COLOR);
  textbackground (ERROR_BACK_COLOR);
  draw_textbox_center (strlen(buffer) + 6, 10, title, ERROR_TEXT_COLOR);

  textcolor (ERROR_TEXT_COLOR);
  hide_mouse();
  cprintf_cent (-2, "%s", buffer);
  show_mouse();
  get_ok_button (26);

	/* Reset screen colors */
  textcolor (orig_text_color);
  textbackground (orig_back_color);

	/* Restore background */
  put_screen (&cap_text);
  delete buffer;
 }
/*========== End of procedure error_box() =================================*/


/*========== Gets a portion of the screen =================================*/
void get_screen (const int width, const int height, CAPTURE_TEXT *text) {
  hide_mouse();
  get_screen ((get_screenwidth() - width)/2 + 1, (get_screenheight() - height)/2 + 1,
	      (get_screenwidth() + width)/2, (get_screenheight() + height)/2, text);
  show_mouse();
 }
/*========== End of procedure get_screen() ================================*/


/*========== Gets a portion of the screen =================================*/
void get_screen (const int x, const int y, const int x1, const int y1, CAPTURE_TEXT *text) {
  text->x = x;
  text->y = y;
  text->x1 = x1;
  text->y1 = y1;

  if (!(text->save_screen = new char[(x1 - x + 2) * (y1 - y + 2) * 2 + 1]))
    bug (MEM_ERR_MSG, "get_screen() - *save_screen (%d)", (x1 - x + 2) * (y1 - y + 2) * 2 + 1);

  hide_mouse();
  gettext (x, y, x1, y1, text->save_screen);
  show_mouse();
 }
/*========== End of procedure get_screen() ================================*/


/*========== Displays and input type box ==================================*/
boolean get_input (char far *string, const int max_length, const int width, const int height, const char *title, const char *msg, ...) {
  int orig_attrib = get_textattr();
  char *buffer;		/* Temp buffer for holding printed string */
  va_list arg_list;	/* For displaying the variable argument list */
  boolean ret;
  CAPTURE_TEXT cap_text;

	/* Attempt to store screen background */
  get_screen (width, height, &cap_text);

	/* Allocated temp buffer and print msg string to it */
  if (!(buffer = new char [MAX_STRING_LENGTH + 1]))
    bug (MEM_ERR_MSG, "error_box() - *buffer (%d)", MAX_STRING_LENGTH);

	/* Parse the parameters */
  va_start (arg_list, msg);
  vsprintf (buffer, msg, arg_list);
  va_end (arg_list);

	/* Draw the box */
  textcolor (BIGWIN_BORD_COLOR);
  textbackground (BIGWIN_BACK_COLOR);
  draw_textbox_center (width, height, title, BIGWIN_TEXT_COLOR);

  hide_mouse();
  textcolor (BIGWIN_TEXT_COLOR);
  cprintf_cent (-1, "%s", buffer);

  textcolor (LIGHTGRAY);
  textbackground (BLACK);
  show_mouse();

  ret = input_string (string, 40 - max_length/2 - 1, get_screenheight()/2 + 1, max_length);
  put_screen (&cap_text);

  delete buffer;	/* Delete temp buffer */
  textattr (orig_attrib);
  return (ret);
 }
/*========== End of function get_input() ==================================*/


/*========== Displays an Message Dialogue Type box ========================*/
void get_message_box (const char *title, const char *msg, ...) {
  int orig_text_color, orig_back_color;	/* Save current color settings */
  char *buffer;		/* Temp buffer for holding printed string */
  CAPTURE_TEXT cap_text;	/* For saving the screen background */
  va_list arg_list;	/* For displaying the variable argument list */

	/* Allocated temp buffer and print msg string to it */
  if (!(buffer = new char [MAX_STRING_LENGTH + 1]))
    bug (MEM_ERR_MSG, "error_box() - *buffer (%d)", MAX_STRING_LENGTH);

  va_start (arg_list, msg);
  vsprintf (buffer, msg, arg_list);
  va_end (arg_list);

  get_screen (strlen(buffer) + 6, 10, &cap_text);

  orig_text_color = get_text_color();
  orig_back_color = get_back_color();

  textcolor (BIGWIN_BORD_COLOR);
  textbackground (BIGWIN_BACK_COLOR);
  draw_textbox_center (strlen(buffer) + 6, 10, title, BIGWIN_TEXT_COLOR);

  hide_mouse();
  textcolor (BIGWIN_TEXT_COLOR);
  cprintf_cent (-2, "%s", buffer);
  show_mouse();
  get_ok_button (26);

	/* Reset screen colors */
  textcolor (orig_text_color);
  textbackground (orig_back_color);

	/* Restore background */
  put_screen (&cap_text);
  delete buffer;
 }
/*========== End of procedure error_box() =================================*/


/*========== Displays one OK button on Screen and Gets Input till pressed =*/
void get_ok_button(const int y) {

  if (get_screenheight() == 25)
    ok_button.y = y;	/* Set new position for button */
  else
    ok_button.y50 = y;

  ok_button.draw ();

  do {
    switch (ok_button.check (MOUSEX, MOUSEY, get_mouse_event(LEFT_BUTTON)) ) {
      case TRUE: return;
     }

    GetKey();
  } while (!get_enter_state() && !get_esc_state());

 }
/*========== End of procedure get_ok_button() =============================*/


/*========== Displays a Y/N/ESC Dialogue Box ==============================*/
boolean get_yn_choice(const char *title, const char *msg, ...) {
  char *buffer;		/* Temp buffer for holding printed string */
  va_list arg_list;	/* For displaying the variable argument list */
  boolean ret;

	/* Allocated temp buffer and print msg string to it */
  if (!(buffer = new char [MAX_STRING_LENGTH + 1]))
    bug (MEM_ERR_MSG, "error_box() - *buffer (%d)", MAX_STRING_LENGTH);

  va_start (arg_list, msg);
  vsprintf (buffer, msg, arg_list);
  va_end (arg_list);

  ret = get_yn_choice (strlen(buffer) + 10, 9, title, buffer);
  delete buffer;
  return (ret);
 }
/*========== End of function get_yn_choice() ==============================*/


/*========== Displays a Y/N/ESC Dialogue Box ==============================*/
boolean get_yn_choice(const int width, const height, const char *title, const char *msg) {
  int orig_text_color, orig_back_color;	/* Save current color settings */

  orig_text_color = get_text_color();
  orig_back_color = get_back_color();

  textcolor (BIGWIN_BORD_COLOR);
  textbackground (BIGWIN_BACK_COLOR);
  draw_textbox_center (width, height, title, BIGWIN_TEXT_COLOR);

  hide_mouse();
  textcolor (BIGWIN_TEXT_COLOR);
  cprintf_cent (-2, "%s [y/n]?", msg);
  show_mouse();

  yes_no_buttons.draw ();

  do {		//Get input
    switch (yes_no_buttons.check (MOUSEX, MOUSEY, get_mouse_event(LEFT_BUTTON))) {
      case 0: textcolor (orig_text_color);                /* Yes, reset colors 1st */
	      textbackground (orig_back_color);
	      return(TRUE);
      case 1: textcolor (orig_text_color);                /* No, reset colors 1st */
	      textbackground (orig_back_color);
	      return(FALSE);
     } //End of switch which button pressed

    GetKey();	/* Get keyboard input */
  } while (toupper(keylast) != 'Y' && toupper(keylast) != 'N' && !get_esc_state());

  delay(KEY_DELAY);

  textcolor (orig_text_color);                /* Reset colors */
  textbackground (orig_back_color);

  if (toupper(keylast) == 'Y') return(TRUE);
  return(FALSE); }
/*========== End of function get_yn_choice() ==============================*/


/*========== Initializes the Default Button Arrays ========================*/
void init_buttons (void) {
  yes_no_buttons.new_button (28, 25, 13, 7, 3, "Yes");
  yes_no_buttons.new_button (45, 25, 13, 7, 3, "No");
  dos_err_buttons.new_button (23, 26, 13, 9, 3, "Retry", WHITE, BLACK, LIGHTGRAY);
  dos_err_buttons.new_button (36, 26, 13, 9, 3, "Abort", WHITE, BLACK, LIGHTGRAY);
  dos_err_buttons.new_button (49, 26, 13,10, 3, "Ignore", WHITE, BLACK, LIGHTGRAY);
  ok_button.init (37, 26, 14, 6, 3, "Ok", BLACK, RED, CYAN);
  atexit (destroy_buttons);
 }
/*========== End of procedure init_buttons() ==============================*/


/*========== Reads a string from keyboard using KEY16 library =============*/
boolean input_string (char *string, int x, int y, int max_length) {
  boolean done = FALSE, redraw = TRUE;
  char buffer[81];

  strcpy(buffer, string);

  while (!done) {
    if (redraw) {  /* Update the inputted text */
      hide_mouse();
      _setcursortype(_NOCURSOR);
      redraw = FALSE;
      gotoxy(x, y);
      cprintf (" %-*s ", max_length, buffer);
      gotoxy(x + strlen(buffer) + 1, y);
      _setcursortype(_NORMALCURSOR);
      show_mouse();
     }

    keylast = 0;

    while (keylast == 0) {   /* Input loop */
      GetKey();
     }

    if (get_esc_state()) { /* ESC */
      _setcursortype(_NOCURSOR);
      return(FALSE);
     }
    else if (key_press[SCAN_BKSP] != 0 && strlen(buffer) > 0) { /* Backspace */
      buffer[strlen(buffer) - 1] = '\0';
      redraw = TRUE;
      delay(KEY_DELAY);
     }
    else if (get_enter_state()) { /* Enter */
      done = TRUE;
      delay(KEY_DELAY);
     }
    else if (isprintable((char)keylast) && strlen(buffer) < max_length) { //Add character to end of string
      chrcat(buffer, (char)keylast);
      redraw = TRUE;
      delay(KEY_DELAY);
     }
   } /* End of while loop... */

  strcpy (string, buffer);  /* Make sure we return the inputted string */
  _setcursortype(_NOCURSOR);
  return(TRUE); }
/*========== End of function input_string() ===============================*/


/*========== Puts a previously saved portion of the screen ================*/
void put_screen (CAPTURE_TEXT *text) {
  hide_mouse();
  puttext (text->x, text->y, text->x1, text->y1, text->save_screen);
  show_mouse();
  delete text->save_screen;
 }
/*========== End of procedure put_screen() ================================*/



/*========== Waits for the user to press a key ============================*/
unsigned char wait_for_key(void) {
  ClearKeys();

  do {
    GetKey();
  } while (keylast == 0);

  return (keylast); }
/*========== End of function wait_for_key() ===============================*/
